@@ -12,17 +12,15 @@
 #    under the License.
 
 import argparse
-import distutils.version as dist_version
+import io
 import re
 import sys
+from unittest import mock
 
 import ddt
 import fixtures
 from keystoneauth1 import fixture
-import mock
-import prettytable
 import requests_mock
-import six
 from testtools import matchers
 
 from novaclient import api_versions
@@ -212,7 +210,7 @@ class DeprecatedActionTest(utils.TestCase):
         action.assert_called_once_with(
             'option_strings', 'dest', help='Deprecated', a=1, b=2, c=3)
 
-    @mock.patch.object(sys, 'stderr', six.StringIO())
+    @mock.patch.object(sys, 'stderr', io.StringIO())
     def test_get_action_nolookup(self):
         action_class = mock.Mock()
         parser = mock.Mock(**{
@@ -230,7 +228,7 @@ class DeprecatedActionTest(utils.TestCase):
         self.assertFalse(action_class.called)
         self.assertEqual(sys.stderr.getvalue(), '')
 
-    @mock.patch.object(sys, 'stderr', six.StringIO())
+    @mock.patch.object(sys, 'stderr', io.StringIO())
     def test_get_action_lookup_noresult(self):
         parser = mock.Mock(**{
             '_registry_get.return_value': None,
@@ -248,7 +246,7 @@ class DeprecatedActionTest(utils.TestCase):
                          'WARNING: Programming error: Unknown real action '
                          '"store"\n')
 
-    @mock.patch.object(sys, 'stderr', six.StringIO())
+    @mock.patch.object(sys, 'stderr', io.StringIO())
     def test_get_action_lookup_withresult(self):
         action_class = mock.Mock()
         parser = mock.Mock(**{
@@ -267,7 +265,7 @@ class DeprecatedActionTest(utils.TestCase):
             'option_strings', 'dest', help='Deprecated', const=1)
         self.assertEqual(sys.stderr.getvalue(), '')
 
-    @mock.patch.object(sys, 'stderr', six.StringIO())
+    @mock.patch.object(sys, 'stderr', io.StringIO())
     @mock.patch.object(novaclient.shell.DeprecatedAction, '_get_action')
     def test_call_unemitted_nouse(self, mock_get_action):
         obj = novaclient.shell.DeprecatedAction(
@@ -282,7 +280,7 @@ class DeprecatedActionTest(utils.TestCase):
         self.assertEqual(sys.stderr.getvalue(),
                          'WARNING: Option "option_string" is deprecated\n')
 
-    @mock.patch.object(sys, 'stderr', six.StringIO())
+    @mock.patch.object(sys, 'stderr', io.StringIO())
     @mock.patch.object(novaclient.shell.DeprecatedAction, '_get_action')
     def test_call_unemitted_withuse(self, mock_get_action):
         obj = novaclient.shell.DeprecatedAction(
@@ -298,7 +296,7 @@ class DeprecatedActionTest(utils.TestCase):
                          'WARNING: Option "option_string" is deprecated; '
                          'use this instead\n')
 
-    @mock.patch.object(sys, 'stderr', six.StringIO())
+    @mock.patch.object(sys, 'stderr', io.StringIO())
     @mock.patch.object(novaclient.shell.DeprecatedAction, '_get_action')
     def test_call_emitted_nouse(self, mock_get_action):
         obj = novaclient.shell.DeprecatedAction(
@@ -313,7 +311,7 @@ class DeprecatedActionTest(utils.TestCase):
             'parser', 'namespace', 'values', 'option_string')
         self.assertEqual(sys.stderr.getvalue(), '')
 
-    @mock.patch.object(sys, 'stderr', six.StringIO())
+    @mock.patch.object(sys, 'stderr', io.StringIO())
     @mock.patch.object(novaclient.shell.DeprecatedAction, '_get_action')
     def test_call_emitted_withuse(self, mock_get_action):
         obj = novaclient.shell.DeprecatedAction(
@@ -393,8 +391,8 @@ class ShellTest(utils.TestCase):
         orig = sys.stdout
         orig_stderr = sys.stderr
         try:
-            sys.stdout = six.StringIO()
-            sys.stderr = six.StringIO()
+            sys.stdout = io.StringIO()
+            sys.stderr = io.StringIO()
             _shell = novaclient.shell.OpenStackComputeShell()
             _shell.main(argstr.split())
         except SystemExit:
@@ -433,7 +431,7 @@ class ShellTest(utils.TestCase):
         if required is None:
             required = [
                 '.*?^usage: ',
-                '.*?^\s+set-password\s+Change the admin password',
+                '.*?^\\s+set-password\\s+Change the admin password',
                 '.*?^See "nova help COMMAND" for help on a specific command',
             ]
         stdout, stderr = self.shell(command)
@@ -451,6 +449,9 @@ class ShellTest(utils.TestCase):
     def test_help_no_options(self):
         self._test_help('')
 
+    def test_help_no_subcommand(self):
+        self._test_help('--os-compute-api-version 2.87')
+
     def test_help_on_subcommand(self):
         required = [
             '.*?^usage: nova set-password',
@@ -580,20 +581,13 @@ class ShellTest(utils.TestCase):
     def test_password(self, mock_getpass, mock_stdin, m_requests):
         mock_stdin.encoding = "utf-8"
 
-        # default output of empty tables differs depending between prettytable
-        # versions
-        if (hasattr(prettytable, '__version__') and
-                dist_version.StrictVersion(prettytable.__version__) <
-                dist_version.StrictVersion('0.7.2')):
-            ex = '\n'
-        else:
-            ex = '\n'.join([
-                '+----+------+--------+------------+-------------+----------+',
-                '| ID | Name | Status | Task State | Power State | Networks |',
-                '+----+------+--------+------------+-------------+----------+',
-                '+----+------+--------+------------+-------------+----------+',
-                ''
-            ])
+        ex = '\n'.join([
+            '+----+------+--------+------------+-------------+----------+',
+            '| ID | Name | Status | Task State | Power State | Networks |',
+            '+----+------+--------+------------+-------------+----------+',
+            '+----+------+--------+------------+-------------+----------+',
+            ''
+        ])
         self.make_env(exclude='OS_PASSWORD')
         self.register_keystone_discovery_fixture(m_requests)
         stdout, stderr = self.shell('list')
@@ -621,26 +615,32 @@ class ShellTest(utils.TestCase):
                           self._test_service_type,
                           'unknown', 'compute', self.mock_client)
 
-    @mock.patch('sys.argv', ['nova'])
-    @mock.patch('sys.stdout', six.StringIO())
-    @mock.patch('sys.stderr', six.StringIO())
+    @mock.patch('sys.stdout', io.StringIO())
+    @mock.patch('sys.stderr', io.StringIO())
     def test_main_noargs(self):
         # Ensure that main works with no command-line arguments
         try:
-            novaclient.shell.main()
+            novaclient.shell.main([])
         except SystemExit:
             self.fail('Unexpected SystemExit')
 
         # We expect the normal usage as a result
-        self.assertIn('Command-line interface to the OpenStack Nova API',
-                      sys.stdout.getvalue())
+        self.assertIn(
+            'Command-line interface to the OpenStack Nova API',
+            sys.stdout.getvalue(),
+        )
+        # We also expect to see the deprecation warning
+        self.assertIn(
+            'nova CLI is deprecated and will be a removed in a future release',
+            sys.stderr.getvalue(),
+        )
 
     @mock.patch.object(novaclient.shell.OpenStackComputeShell, 'main')
     def test_main_keyboard_interrupt(self, mock_compute_shell):
         # Ensure that exit code is 130 for KeyboardInterrupt
         mock_compute_shell.side_effect = KeyboardInterrupt()
         try:
-            novaclient.shell.main()
+            novaclient.shell.main([])
         except SystemExit as ex:
             self.assertEqual(ex.code, 130)
 
@@ -761,11 +761,17 @@ class ShellTest(utils.TestCase):
     def test_main_error_handling(self, mock_compute_shell):
         class MyException(Exception):
             pass
-        with mock.patch('sys.stderr', six.StringIO()):
+        with mock.patch('sys.stderr', io.StringIO()):
             mock_compute_shell.side_effect = MyException('message')
-            self.assertRaises(SystemExit, novaclient.shell.main)
+            self.assertRaises(SystemExit, novaclient.shell.main, [])
             err = sys.stderr.getvalue()
-        self.assertEqual(err, 'ERROR (MyException): message\n')
+        # We expect to see the error propagated
+        self.assertIn('ERROR (MyException): message\n', err)
+        # We also expect to see the deprecation warning
+        self.assertIn(
+            'nova CLI is deprecated and will be a removed in a future release',
+            err,
+        )
 
 
 class TestLoadVersionedActions(utils.TestCase):
